/*
 * UNC0xx firmware. (c) Unicontrollers, Ltd., 2010-2013
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file crc8.c
 *
 * Written by Andrey V. Stolyarov
 */

#include <stdint.h>
#include "crc8.h"


enum { crc8polynomial = 0x18 }; /* 0X18 = X^8+X^5+X^4+X^0 */

uint8_t crc8_next_byte(uint8_t crc, uint8_t data)
{
    uint8_t bits_left = 8;
    uint8_t flag;
    do {
        flag = (crc ^ data) & 0x01;
        if(flag)
            crc ^= crc8polynomial;
        crc >>= 1;
        if(flag)
            crc |= 0x80;
        data >>= 1;
    } while(--bits_left);
    return crc;
}

uint8_t crc8(uint8_t *data_in, uint8_t data_len)
{
    uint8_t crc = crc8_init();
    while(data_len) {
        crc = crc8_next_byte(crc, *data_in);
        data_in++;
        data_len--;
    }
    return crc;
}
