/*
 * UNC001 firmware. (c) Unicontrollers, Ltd., 2010-2013
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file sernum.c
 * Implements the (public) function usbFunctionDescriptor, which
 * presently only able to transmit the Serial Number.  The Number
 * has to be represented in a very ugly way to be used by the USB
 * driver; this module does all this.
 * In case the SUPPORT_SERNUM_DEBINFO macro is defined, there are
 * also two functions that allow to add some debug information
 * to the serial number string.  If the macro is not defined, these
 * functions are replaced with macro placeholders (see sernum.h)
 *
 * Written by Andrey V. Stolyarov
 *
 */

#include <avr/eeprom.h>
#include "usbdrv.h"
#include "misc.h"

#include "sernum.h"

#ifdef SUPPORT_SERNUM_DEBINFO
#  define SERNUM_LEN 25
#else
#  define SERNUM_LEN 14
#endif

/* please note that this implementation has proven to be smaller than
   if we just allocate the array and then initialize it's [0], [5] and [10]
   elements
 */
static int sernum[SERNUM_LEN+1] = { 
    USB_STRING_DESCRIPTOR_HEADER(SERNUM_LEN), /* one integer */
    '0','0','0','0',':', 
    '0','0','0','0',':',
    '0','0','0','0'
#ifdef SUPPORT_SERNUM_DEBINFO
    ,':','%','%','%','%','%','%','%','%','%','%'
#endif
};

#if 0
static void fill_hex_word_from_eeprom(uint8_t eeprom_pos, int *dest)
{
    uint8_t b;
    b = eeprom_read_byte(((uint8_t*)0) + eeprom_pos);
    dest[2] = hexdigit((b & 0xf0) >> 4);
    dest[3] = hexdigit(b & 0x0f);
    b = eeprom_read_byte(((uint8_t*)0) + eeprom_pos + 1);
    dest[0] = hexdigit((b & 0xf0) >> 4);
    dest[1] = hexdigit(b & 0x0f);
}
static void sernum_init()
{
    fill_hex_word_from_eeprom(0, sernum + 1);
    fill_hex_word_from_eeprom(2, sernum + 6);
    fill_hex_word_from_eeprom(4, sernum + 11);
}
#endif

static void sernum_init()
{
    static const uint8_t sernum_offsets[] = { 3, 1, 8, 6, 13, 11 };
    int8_t i;
    uint8_t b;
    uint8_t k;
    for(i = 5; i >= 0; i--) {
        b = eeprom_read_byte(((uint8_t*)0) + i);
        k = sernum_offsets[i];
        sernum[k] = hexdigit((b & 0xf0) >> 4);
        sernum[k+1] = hexdigit(b & 0x0f);
    }
}

#ifdef SUPPORT_SERNUM_DEBINFO
void set_sernum_debug_info(const char *info)
{
    uint8_t i;
    for(i=16; i<=25; i++) {
        if(*info) {
             sernum[i] = *info;
             info++;
        } else {
             sernum[i] = '%';
        }
    }
}
void set_sernum_debug_info2(uint16_t n, const char *info)
{
    uint8_t i;
    for(i=16; i<=21; i++) {
        if(*info) {
             sernum[i] = *info;
             info++;
        } else {
             sernum[i] = '%';
        }
    }
    for(i=0; i<=3; i++) {
        sernum[i+22] = hexdigit( (n>>(4*(3-i))) & 0x0f );
    }
}
#endif

USB_PUBLIC usbMsgLen_t usbFunctionDescriptor(struct usbRequest *rq)
{
    if(rq->wValue.bytes[1] == 3) { /* string */ 
        if(rq->wValue.bytes[0] == 3) { /* serial number */
           sernum_init();
           usbMsgPtr = (uchar*) sernum;
           return sizeof(sernum);
        }
    }
    return 0;
}
