/*
 * UNC001 firmware. (c) Unicontrollers, Ltd., 2010-2013
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file unc.c
 * The main file for the firmware implementation.
 *
 * Written by Andrey V. Stolyarov
 *
 */


#include <avr/wdt.h>
#include <avr/sleep.h> 
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */

#include <avr/pgmspace.h>   /* required by usbdrv.h */

#include "unc_1w.h"

#include "usbdrv.h"

#include "ucd.h"
#include "unc_prim.h"
#include "rs485.h"
#include "usb_io.h"
#include "print.h"
#include "led.h"

static uint8_t timer_intr_flag = 0;

ISR(TIMER1_COMPA_vect)
{
    TCNT1=0;
    timer_intr_flag = 1;
}

void main() __attribute__ ((noreturn));
void main()
{
    uint8_t i;

    /* init UNC-specific stuff first */
    init_primary_hardware();
    /* init_global_state(); // now done by init_prim_hw */ 
    init_led();
    init_rs485();

    //wdt_enable(WDTO_1S);
    wdt_disable();  /* disable the watchdog timer for safety */
    /* RESET status: all port bits are inputs without pull-up.
     * That's the way we need D+ and D-. Therefore we don't need any
     * additional hardware initialization.
     */
    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration,
                               do this while interrupts are disabled! */
#if 1  /* no gain at all... WTF?! */
    i = 0;
    while(--i){             /* fake USB disconnect for > 250 ms */
        //wdt_reset();
        _delay_ms(1);
    }
#else
    _delay_ms(255);
#endif
    usbDeviceConnect();

    unc1w_init();

    sei();

    startup_hook();

    for(;;){                /* main event loop */
        //wdt_reset();
        timer_intr_flag = 0;
        sleep_mode();

        usbPoll();
        process_usb_interrupt_input();
        process_serial_input();

        /* the rest is tick-based; skip if there wasn't a tick */
        if(!timer_intr_flag)
            continue;
        ++global_state.timemark;
        led_hook();
        do_input_confirmation();
        process_delayed_actions();
        unc1w_hook();
        rs485_hook();
    }
}

