/*
 * UNC0xx GUI-based software. (c) Unicontrollers, Ltd., 2012
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file demo.cpp
 * Small GUI demo program for a UNC011 device with UNC010/in and UNC010/out
 *
 * Written by Andrey V. Stolyarov
 *
 */

#include <wx/app.h>
#include <wx/frame.h>
#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/button.h>
#include <wx/checkbox.h>
#include <wx/timer.h>
#include <wx/gbsizer.h>
#include <wx/image.h>
#include <wx/statbmp.h>
#include <wx/mstream.h>

#include <stdio.h>

#include "demo_dev.hpp"

#include "_unc_logo.inc"

class UNCDemo : public wxFrame {
    wxButton *main_outs[4];
    wxWindow *main_outs_indicators[4];
    wxButton *ext_outs[4];
    wxWindow *ext_outs_indicators[4];

    static const wxColour *main_outs_colours[4];
    static const wxColour *ext_outs_colours[4];

    wxWindow *input_indicators[4];
    wxStaticText *temperature;
    wxStaticText *tabletcode;

    wxButton *fun_buttons[3];

    wxStaticText *dev_stat;

    bool device_active;
public:
    UNCDemo(const char *title)
        : wxFrame(0, wxID_ANY, wxString(title, wxConvLocal),
          wxDefaultPosition, wxDefaultSize,
          wxDEFAULT_FRAME_STYLE|wxWANTS_CHARS),
          device_active(false)
    {
        int i;

        wxGridBagSizer* gridBagSizer = new wxGridBagSizer(10,10);
        //gridBagSizer->SetEmptyCellSize(wxSize(100,100));
        gridBagSizer->SetRows(10);
        gridBagSizer->SetCols(6);
        SetSizer(gridBagSizer);

        for(i = 1; i < 6; i++)
            gridBagSizer->AddGrowableCol(i, 1);
        gridBagSizer->SetFlexibleDirection(wxBOTH);

        wxBoxSizer *top_row = new wxBoxSizer(wxHORIZONTAL);

        wxImage::AddHandler(new wxPNGHandler);
        wxMemoryInputStream strm(unc_logo, sizeof(unc_logo));
        wxImage img;
        img.LoadFile(strm, wxBITMAP_TYPE_PNG);
        wxBitmap bmp(img);
        top_row->Add(
            new wxStaticBitmap(this, wxID_ANY, bmp,
                wxDefaultPosition, wxSize(150, 150)));

        top_row->AddStretchSpacer();

        wxStaticText *titletext = new wxStaticText(this, wxID_ANY,
            wxT("UNICONTROLLERS LTD\nUNC01x DEMO"),
            wxDefaultPosition, wxDefaultSize,
            wxALIGN_CENTER|wxST_NO_AUTORESIZE);
        titletext->SetFont(wxFont(35, wxDEFAULT, wxFONTSTYLE_ITALIC, wxBOLD));
        top_row->Add(titletext, 0, wxEXPAND);

        top_row->AddStretchSpacer();

        dev_stat = new wxStaticText(this, wxID_STATIC, wxT("not connected"));
        dev_stat->SetFont(wxFont(14, wxDEFAULT, wxFONTSTYLE_ITALIC, wxNORMAL));
        top_row->Add(dev_stat, 0, wxEXPAND);

        top_row->AddStretchSpacer();

        gridBagSizer->Add(top_row, wxGBPosition(0, 0), wxGBSpan(1, 6), wxGROW);

        for(i = 0; i<4; i++) {
            main_outs[i] = new wxButton(this, 1100+i, wxT("RELAY"));
            gridBagSizer->Add(main_outs[i], wxGBPosition(7, i+1),
                wxDefaultSpan, wxGROW);
            main_outs_indicators[i] = new wxWindow(this, wxID_ANY);
            main_outs_indicators[i]->SetBackgroundColour(*wxLIGHT_GREY);
            gridBagSizer->Add(main_outs_indicators[i], wxGBPosition(6, i+1),
                wxDefaultSpan, wxGROW);
        }
        gridBagSizer->AddGrowableRow(6);

        for(i = 0; i<4; i++) {
                    // XXX 1104 works for 4xxxxxx versions of firmware
                    // for older versions, 1108 must be specified here
            ext_outs[i] = new wxButton(this, 1104+i, wxT("EXT.LINE"));
            gridBagSizer->Add(ext_outs[i], wxGBPosition(2, i+1),
                wxDefaultSpan, wxGROW);
            ext_outs_indicators[i] = new wxWindow(this, wxID_ANY);
            ext_outs_indicators[i]->SetBackgroundColour(*wxLIGHT_GREY);
            gridBagSizer->Add(ext_outs_indicators[i], wxGBPosition(1, i+1),
                wxDefaultSpan, wxGROW);
        }
        ext_outs[3]->Disable();
        gridBagSizer->AddGrowableRow(1);

        for(int i = 0; i<4; i++) {
            input_indicators[i] = new wxWindow(this, wxID_ANY);
            input_indicators[i]->SetBackgroundColour(*wxBLACK);
            gridBagSizer->Add(input_indicators[i], wxGBPosition(3, i+1),
                wxDefaultSpan, wxGROW);
        }
        gridBagSizer->AddGrowableRow(3);

        temperature = new wxStaticText(this, wxID_STATIC, wxT(""));
        temperature->SetFont(wxFont(20, wxDEFAULT, wxFONTSTYLE_ITALIC,wxBOLD));
        gridBagSizer->Add(temperature, wxGBPosition(4, 1), wxGBSpan(1, 4));
        tabletcode = new wxStaticText(this, wxID_STATIC, wxT(""));
        tabletcode->SetFont(wxFont(20, wxDEFAULT, wxFONTSTYLE_ITALIC, wxBOLD));
        gridBagSizer->Add(tabletcode, wxGBPosition(5, 1), wxGBSpan(1, 4));
        //gridBagSizer->AddGrowableRow(4);
        //gridBagSizer->AddGrowableRow(5);


        gridBagSizer->Add(new wxStaticText(this, wxID_STATIC, wxT("")),
                          wxGBPosition(8, 5));
        wxButton *qb;
        gridBagSizer->Add((qb = new wxButton(this, wxID_EXIT, wxT("Quit"))),
                          wxGBPosition(8, 4), wxDefaultSpan, wxGROW);

        wxStaticText *copytext = new wxStaticText(this, wxID_STATIC,
            wxT("Copyright (c) Unicontrollers, Ltd., 2012.  "
                "Free software (GNU GPL v.3).\n"
                "See http://www.unicontrollers.com for details"),
            wxDefaultPosition, wxDefaultSize,
                wxALIGN_CENTER|wxST_NO_AUTORESIZE);
        copytext->SetFont(wxFont(14, wxDEFAULT, wxFONTSTYLE_ITALIC, wxNORMAL));
        gridBagSizer->Add(copytext, wxGBPosition(9,0), wxGBSpan(1,6), wxGROW);

        wxBoxSizer *fun_box = new wxBoxSizer(wxVERTICAL);
        for(i = 0; i<3; i++) {
            wxString s;
            s.Printf(wxT("press me\nFUNC %d\npress me"), i+1);
            fun_buttons[i] = new wxButton(this, 1120+i, s);
            fun_box->Add(fun_buttons[i], 1, wxEXPAND);    
        }
        gridBagSizer->Add(fun_box, wxGBPosition(1,5), wxGBSpan(6,1), wxGROW);


        qb->SetFocus();
            // for an unknown reason it is insufficient to focus
            // on the Frame itself
        Refresh();
        //gridBagSizer->Fit(this);
        gridBagSizer->SetSizeHints(this);
#ifndef _WIN32
        ShowFullScreen(true, wxFULLSCREEN_ALL);
#else
        Maximize(true);
        Show(true);
#endif
    } 
    void DeviceActive() {
        if(device_active) return;
        device_active = true;
        dev_stat->SetLabel(wxT("connected"));
    }
    void UpdateState(int state) {
        DeviceActive();
        for(int i = 0; i<4; i++) {
            input_indicators[i]->
                SetBackgroundColour(
                    (state>>(8 + i))&1 ? *wxGREEN : *wxBLACK);
            main_outs_indicators[i]->
                SetBackgroundColour(
                    (state>>i)&1 ? *main_outs_colours[i] : *wxLIGHT_GREY);
            ext_outs_indicators[i]->
                SetBackgroundColour(
                    (state>>(12+i))&1 ? *ext_outs_colours[i] : *wxLIGHT_GREY);
// XXX (12+i) will work provided that SLOT#2 is output
        }
        Refresh();
    }
    void UpdateTemp(const char *temp) {
        DeviceActive();
        if(*temp) {
            wxString t(wxT("t: "));
            t += wxString(temp, wxConvLocal);
            temperature->SetLabel(t);
        } else {
            temperature->SetLabel(wxT(""));
        }
        temperature->Refresh();
    }
    void UpdateTablet(const char *tbl) {
        DeviceActive();
        if(*tbl) {
            wxString t(wxT("iB: "));
            t += wxString(tbl, wxConvLocal);
            tabletcode->SetLabel(t);
        } else {
            tabletcode->SetLabel(wxT(""));
        }
        tabletcode->Refresh();
    }
    void DeviceLost() {
        if(!device_active) return;
        UpdateState(0);
        UpdateTemp("");
        UpdateTablet("");
        dev_stat->SetLabel(wxT("not connected"));
        Refresh();
        device_active = false;
    }
};

// the byte seq. is actually BLUE GREEN RED
wxColour col_magenta(0xff00ff);
wxColour col_cyan   (0xffff00);
wxColour col_yellow (0x00ffff);
wxColour col_aquamarine(0xd4ff7f);
wxColour col_belge(0xdcf5f5);
wxColour col_bisque(0xc4e4ff);
wxColour col_hotpink(0xb469ff);


const wxColour *UNCDemo::main_outs_colours[4] = 
    { wxBLUE, wxRED, &col_yellow, &col_hotpink };
const wxColour *UNCDemo::ext_outs_colours[4] =
    { &col_belge, wxGREEN, &col_aquamarine, &col_bisque };

class UNCDemoApp : public wxApp
{
    UNCDemo *demo;
    UNCDevice *the_device;
public:
    UNCDemoApp(UNCDevice *adev) : demo(0), the_device(adev) {}
    virtual bool OnInit() {
        demo = new UNCDemo("UNCDemo");
        //demo->Show(true);
        return true;
    }
    void OnQuit(wxCommandEvent& event) { if(demo) demo->Close(true); }
    void OnKeypress(wxKeyEvent& e) {
        if(demo && e.GetKeyCode() == 'Q') demo->Close(true);
    }
    UNCDemo *GetWindow() const { return demo; }
    void OnOutputButton(wxCommandEvent& event)
    {
        int id = event.GetId();
        if(id >= 1100 && id < 1120) {
            // this is an output button event
            if(the_device)
                the_device->ToggleLine(id - 1100);
        } else
        if(id >= 1120 && id < 1130) {
            // this is an extra function
            int reg = 47 - (id - 1120); // 47, 46, 45, ...
            reg = (reg & 0xff) | 0xf000;
            if(the_device)
                the_device->PerformAction(reg);
        } else {
            event.Skip();
        }
    }

private:
    DECLARE_EVENT_TABLE();
};

class PollingTimer : public wxTimer {
    UNCDemoApp *the_app;
    UNCDevice *the_device;
    int interval;
    int last_state;
    bool told;
    char last_temp[32], last_tablet[32];
    int counter;
public:
    PollingTimer(UNCDemoApp *a_app, UNCDevice *a_dev, int ainterv)
        : the_app(a_app), the_device(a_dev), interval(ainterv),
          last_state(0), told(false)
    {
        *last_temp = 0;
        *last_tablet = 0;
    }
    void Go() {
        Start(interval);
    }
    void Notify() {
        int state;
        char temp[32];
        char tablet[32];
        UNCDemo *the_window = the_app->GetWindow();
        if(!the_window) return;
        if(the_device->Poll(&state, temp, tablet)) {
            if(state != last_state) {
                the_window->UpdateState(state);
                told = true;
                last_state = state;
            }
            if(strcmp(last_temp, temp) != 0) {
                the_window->UpdateTemp(temp);
                told = true;
                strcpy(last_temp, temp);
            }
            if(*tablet) {
                counter = 0;
            } else {
                counter++; 
            }
            if(strcmp(last_tablet, tablet) != 0) {
                if(*tablet) {
                    the_window->UpdateTablet(tablet);
                    told = true;
                    strcpy(last_tablet, tablet);
                } else {
                    if(counter >= 25) {
                        the_window->UpdateTablet("");
                        told = true;
                        *last_tablet = 0;
                    }
                }
            }
            if(!told) {
               the_window->UpdateState(state);
               told = true;
            }
        } else {
            the_window->DeviceLost();
            the_device->OpenDevice();
            told = false;
        }
    }
};

BEGIN_EVENT_TABLE(UNCDemoApp, wxApp)
  EVT_BUTTON(wxID_EXIT, UNCDemoApp::OnQuit)
  EVT_BUTTON(wxID_ANY, UNCDemoApp::OnOutputButton)
  EVT_KEY_DOWN(UNCDemoApp::OnKeypress)
END_EVENT_TABLE()

int main(int argc, char **argv)
{
    int dev_id = 0;

    if(argc > 1) {
        char *err;
        dev_id = strtol(argv[1], &err, 16);
        if(!*argv[1] || *err) {
            fprintf(stderr, "Invalid device id [%s]\n", argv[1]);
            return 1;
        }
    }

    UNCDevice dev(dev_id);
    dev.OpenDevice();
    
    UNCDemoApp *app = new UNCDemoApp(&dev);
    wxApp::SetInstance(app);

    PollingTimer *tm = new PollingTimer(app, &dev, 100);
    tm->Go();

    char *argv_for_wx[2];
    int argc_for_wx = 1;
    argv_for_wx[0] = argv[0];
    argv_for_wx[1] = 0;
    wxEntry(argc_for_wx, argv_for_wx);

    return 0;
}
