/*
 * UNC0xx GUI-based software. (c) Unicontrollers, Ltd., 2012
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file devlist.hpp
 * Interface of the device communication subsystem
 *
 * Written by Andrey V. Stolyarov
 *
 */

#ifndef DEVLIST_HPP_SENTRY
#define DEVLIST_HPP_SENTRY

#include "glue.hpp"

class UncDevice : public UncDeviceInterface {
    int id;
    struct unc0xx_devhdl *dev_handle;
    int last_state;
    int minc, maxc;
    int *counters;
public:
    UncDevice(int a_id, struct unc0xx_devhdl *a_dev_handle);
    ~UncDevice();
    void SyncState();

    int Id() const { return id; }
    bool SetLine(int n, bool on);
    void ZeroCounters();
    bool IsAlive() const;
    bool GetRegisters(unsigned long *hw,
                      unsigned long *user,
                      unsigned long reacts[8]); 

    bool SetUserReg(unsigned long user);
    bool SetHardwareConf(unsigned long hw);
    bool SetReactionReg(int line, unsigned long reg);
    bool SaveToEeprom();
    bool RestoreFromEeprom();
    bool RestoreFactoryDefaults();
    bool SetId(int id);

    // new function
    void CheckHardwareConfiguration();
};

class UncDevList : public UncDeviceCollection {
public:
    UncDevList() : UncDeviceCollection() {}
    virtual void Rescan(bool showhidden);
    virtual void Poll();
    virtual UncDeviceInterface *Find(int id);
};

#endif
