/*
 * UNC0xx GUI-based software. (c) Unicontrollers, Ltd., 2012
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file glue.cpp
 * Implementation of the glue classes, which glues the two subsystems 
 * (GUI and device communication) together
 *
 * Written by Andrey V. Stolyarov
 *
 */



#include "glue.hpp"


UncDeviceInterface::~UncDeviceInterface()
{
    if(the_set)
        the_set->Detach();
    the_set = 0;
}

UncDeviceCollection::UncDeviceCollection()
    : first(0), the_ctrlsetcoll(0)
{
}

UncDeviceCollection::~UncDeviceCollection()
{
    Clear();
}

void UncDeviceCollection::Add(UncDeviceInterface *d)
{
    if(first) {
        last->next = new item;
        last = last->next;
    } else {
        first = last = new item;
    }
    last->next = 0;
    last->dev = d;
}

void UncDeviceCollection::Clear()
{
    while(first) {
        item *t = first;
        first = first->next;
        delete t->dev;
        delete t;
    }
}

UncDeviceInterface* UncDeviceCollection::Iterator::Next()
{
    if(p) {
        UncDeviceInterface *r = p->dev;
        p = p->next;
        return r;
    } else
        return 0;
}

//////////////////////////////////////////////////////////////////


UncCtrlsetInterface::UncCtrlsetInterface(UncCtrlsetCollection *acoll, int a_id)
    : id(a_id), the_collection(acoll), the_device(0)
{
    the_collection->Add(this);
}

UncCtrlsetInterface::~UncCtrlsetInterface()
{
    if(the_device) the_device->Detach();
    the_collection->Remove(this);
}

UncCtrlsetCollection::~UncCtrlsetCollection()
{
    Clear();
}

void UncCtrlsetCollection::Add(UncCtrlsetInterface *d)
{
    if(first) {
        last->next = new item;
        last = last->next;
    } else {
        first = last = new item;
    }
    last->next = 0;
    last->dev = d;
}

bool UncCtrlsetCollection::Remove(UncCtrlsetInterface *d)
{
    item **p = &first;
    while(*p) {
        if((*p)->dev == d) {
            item *tmp = *p;
            *p = (*p)->next;
            if(tmp->next == 0) {
                last = first;
                while(last && last->next)
                    last = last->next;
            }
            delete tmp;
            return true;
        }
        p = &( (*p)->next );
    }
    return false;
}

void UncCtrlsetCollection::Clear()
{
    while(first) {
        item *t = first;
        first = first->next;
        delete t->dev;
        delete t;
    }
}

UncCtrlsetInterface* UncCtrlsetCollection::Iterator::Next()
{
    if(p) {
        UncCtrlsetInterface *r = p->dev;
        p = p->next;
        return r;
    } else
        return 0;
}

bool UncCtrlsetCollection::Reconfigure(int id)
{
    for(item *p = first; p; p = p->next) {
        if(p->dev->Id() == id) {
            p->dev->Reconfigure();
            return true;
        }
    }
    return false;
}

