/*
 * UNC0xx GUI-based software. (c) Unicontrollers, Ltd., 2012, 2013
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file glue.hpp
 * Abstract classes that glue the two main subsystems (GUI and
 * device communication) together.
 *
 * Written by Andrey V. Stolyarov
 *
 */

#ifndef GLUE_HPP_SENTRY
#define GLUE_HPP_SENTRY

class UncCtrlsetInterface;
class UncCtrlsetCollection;

class UncDeviceInterface {
protected:
    UncCtrlsetInterface *the_set;
public:
    UncDeviceInterface() : the_set(0) {}
    virtual ~UncDeviceInterface();
    void Attach(UncCtrlsetInterface *i) { the_set = i; }
    void Detach() { the_set = 0; }
    bool IsDisplayed() const { return the_set != 0; }

    virtual void SyncState() = 0;
    virtual bool SetLine(int n, bool on) = 0;
    virtual void ZeroCounters() = 0;
    virtual bool IsAlive() const = 0;

    virtual bool GetRegisters(unsigned long *hw,
                              unsigned long *user,
                              unsigned long reacts[8]) = 0; 

    virtual bool SetUserReg(unsigned long user) = 0;
    virtual bool SetHardwareConf(unsigned long hw) = 0;
    virtual bool SetReactionReg(int line, unsigned long reg) = 0;
    virtual bool SaveToEeprom() = 0;
    virtual bool RestoreFromEeprom() = 0;
    virtual bool RestoreFactoryDefaults() = 0;
    virtual bool SetId(int id) = 0;
};

class UncDeviceCollection {
    friend class Iterator;
    struct item {
        UncDeviceInterface *dev;
        item *next;
    } *first, *last;
protected:
    UncCtrlsetCollection *the_ctrlsetcoll;
public:
    UncDeviceCollection();
    virtual ~UncDeviceCollection();
    void SetCtrlsetCollection(UncCtrlsetCollection *ctrls)
        { the_ctrlsetcoll = ctrls; }

    virtual void Rescan(bool showhidden) = 0;
    virtual void Poll() = 0;

    virtual UncDeviceInterface *Find(int id) = 0;

    class Iterator {
        UncDeviceCollection::item *p;
    public:
        Iterator(UncDeviceCollection *master) : p(master->first) {}
        UncDeviceInterface *Next();
    };
protected:
    void Add(UncDeviceInterface *d);
    void Clear();
};


class UncCtrlsetCollection;

class UncCtrlsetInterface {
    int id;
protected:
    UncCtrlsetCollection *the_collection;
    UncDeviceInterface *the_device;
public:
    UncCtrlsetInterface(UncCtrlsetCollection *acoll, int a_id);
    virtual ~UncCtrlsetInterface();
    void Attach(UncDeviceInterface *i) { the_device = i; }
    void Detach() { the_device = 0; }
    int Id() const { return id; }

    virtual void SetInput(int line, bool state) = 0;
    virtual void SetOutput(int line, bool state) = 0;
    virtual void SetCounter(int line, int value) = 0;
    virtual void SetStatusBar(const char *status) = 0;
    virtual void UpdateConnectedState() = 0;

    virtual void Reconfigure() = 0;
};

class UncCtrlsetCollection {
    friend class Iterator;
    struct item {
        UncCtrlsetInterface *dev;
        item *next;
    } *first, *last;
protected:
    UncDeviceCollection *the_devcoll;
public:
    UncCtrlsetCollection(UncDeviceCollection *a_devcoll)
        : first(0), the_devcoll(a_devcoll) {}
    virtual ~UncCtrlsetCollection();

    virtual UncCtrlsetInterface* Provide(int id) = 0;

    class Iterator {
        UncCtrlsetCollection::item *p;
    public:
        Iterator(UncCtrlsetCollection *master) : p(master->first) {}
        UncCtrlsetInterface *Next();
    };

    void Add(UncCtrlsetInterface *d);
    bool Remove(UncCtrlsetInterface *d); /* removes d from the collection,
                                            does NOT delete it */

    bool Reconfigure(int id); /* the device needs reconfiguration */
    UncDeviceCollection *GetDeviceCollection() const { return the_devcoll; }

    void CallRescan() { the_devcoll->Rescan(false); }
protected:
    void Clear(); /* clears the collection, DELETING all the objects */
};

/* implemented in devlist.cpp */
UncDeviceCollection* CreateDeviceCollection();


#endif
