/*
 * UNC0xx GUI-based software. (c) Unicontrollers, Ltd., 2012
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file gui.hpp
 * Class headers for the GUI application and its main window
 *
 * Written by Andrey V. Stolyarov
 *
 */

#ifndef UNCMON_GUI_HPP_SENTRY
#define UNCMON_GUI_HPP_SENTRY


#include <wx/app.h>
#include <wx/panel.h>
#include <wx/scrolwin.h>

#include "glue.hpp"

enum {
    extra_pad = 5
};

enum {
    wid_localbase     = wxID_HIGHEST+1,
       // here individual buttons go...
       // main dialog
    wid_dismiss = wid_localbase + 3,
    wid_options,
    wid_hwopts,
    wid_zero,

       // hardware dialog
    wid_submit_userreg,
    wid_change_id,
    wid_set_hwconf,
    wid_save_to_eeprom,
    wid_restore_eeprom,
    wid_restore_factory,


       // reaction reg dialog
    wid_action0,
    wid_action1,

       // now for the control groups
    wid_out_button    = wid_localbase + 1000, // thru 1099
    wid_in_alert      = wid_localbase + 1100, // thru 1199

    wid_rreg_button   = wid_localbase + 1200, // thri 1299

};

enum { series_wid = 99 };

class ScrolledDeviceList;

class SingleDevicePanel : public wxPanel, public UncCtrlsetInterface {
    ScrolledDeviceList *the_parent;
    class wxStaticBoxSizer *statbs;
    class wxGridBagSizer *grid;
    class wxToggleButton *outs[12];
    class wxCheckBox *sticky[12];
    class wxStaticText *delims[8];
    class wxStaticText *count[8];
    //class wxPanel *ins[4];
    class wxGauge *ins[8];
    class wxStaticText *innames[8];
    class wxCheckBox *alert[8];
    class wxStaticText *statusbar;
    class wxButton *zero, *options, *hwopts, *dismiss;
public:
    SingleDevicePanel(ScrolledDeviceList *parent, int a_id);

    void SetInput(int line, bool state);
    void SetOutput(int line, bool state);
    void SetCounter(int line, int value);
    void SetStatusBar(const char *status);
    void UpdateConnectedState();
    void Reconfigure();
private:
    void OnOutputButton(wxCommandEvent& event);
    void OnDismiss(wxCommandEvent& event);
    void OnZero(wxCommandEvent& event);
    void OnOptions(wxCommandEvent& event);
    void OnHwOpts(wxCommandEvent& event);

    DECLARE_EVENT_TABLE();
};

 
class ScrolledDeviceList :
    public wxScrolledWindow, public UncCtrlsetCollection
{
    wxSize mins;
    class wxBoxSizer* sizer; // sizers are owned by wxw (no deletion!)
public:
    ScrolledDeviceList(wxWindow* parent, wxWindowID id,
                       class UncDeviceCollection *a_devcoll);
    UncCtrlsetInterface* AddDevice(int id);
    void Rearrange();
    wxSize GetMin() const { return mins; }
    UncCtrlsetInterface* Provide(int id);
};

class UncMonitorGui : public wxApp {
    wxFrame *frame;
    ScrolledDeviceList *list;
    wxButton *adddev;
    wxButton *rescan;
    wxButton *about;
    wxButton *quit;
    class UncDeviceCollection *the_devcoll;
public:
    UncMonitorGui(UncDeviceCollection *a_devcoll)
        : the_devcoll(a_devcoll) {} 
    bool OnInit();
    void OnAdddev(wxCommandEvent& event);
    void OnRescan(wxCommandEvent& event);
    void OnAbout(wxCommandEvent& event);
    void OnQuit(wxCommandEvent& event);

    UncCtrlsetCollection *GetCtrlsetCollection() const { return list; }

private:
    DECLARE_EVENT_TABLE();
};

#endif
